import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.ArcType;
import javafx.stage.Stage;

public class CanvasDemo extends Application {
    private static final double WIDTH = 500.0;
    private static final double HEIGHT = 700.0;
    
    @Override
    public void start(Stage stage) {
        StackPane root = new StackPane();
        
        drawCanvas(root);
        
        Scene scene = new Scene(root, WIDTH, HEIGHT);
        
        stage.setTitle("Canvas Demo");
        stage.setScene(scene);
        stage.show();
    }
    
    private void drawCanvas(StackPane parent) {
        // キャンバスを生成
        Canvas canvas = new Canvas(WIDTH, HEIGHT);
        parent.getChildren().add(canvas);
        
        // グラフィックスコンテキストを取得
        // 描画はグラフィックスコンテキストに対して行う
        GraphicsContext gc = canvas.getGraphicsContext2D();

        // 四角形
        // 塗りつぶしと線の描画は別々に行う
        // 塗りつぶし
        gc.setFill(Color.CYAN);
        gc.fillRect(20.0, 20.0, 200.0, 100.0);
        // 線の描画
        gc.setStroke(Color.DARKBLUE);
        gc.strokeRect(20.0, 20.0, 200.0, 100.0);
        
        // 角丸四角形
        gc.setFill(Color.BLUE);
        gc.fillRoundRect(250.0, 20.0, 200.0, 100.0, 20.0, 20.0);

        // 直線
        gc.setStroke(Color.BLACK);
        gc.strokeLine(20.0, 150.0, 220.0, 250.0);
        
        // パスを使用して3次ベジェ曲線を描画する
        gc.beginPath();
        gc.moveTo(250.0, 150.0);
        gc.bezierCurveTo(250.0, 250.0, 450.0, 150.0, 450.0, 250.0);
        // パスの塗りつぶし
        gc.setFill(Color.RED);
        gc.fill();
        // パスの線の描画
        gc.setStroke(Color.BLACK);
        gc.stroke();

        // 円
        gc.setFill(Color.GREEN);
        gc.fillOval(50.0, 250.0, 100.0, 100.0);
        
        // 楕円
        gc.setFill(Color.GREENYELLOW);
        gc.fillOval(270.0, 250.0, 160.0, 100.0);

        // 円弧
        gc.setFill(Color.ORANGE);
        gc.fillArc(25.0, 400.0, 200.0, 100.0, 60.0, 270.0, ArcType.ROUND);
        gc.setStroke(Color.BLACK);
        gc.strokeArc(25.0, 400.0, 200.0, 100.0, 60.0, 270.0, ArcType.ROUND);

        // 文字列
        gc.setFill(Color.BLACK);
        gc.fillText("Hello, World!", 300.0, 450.0);
        
        // イメージを描画
        Image image = new Image("dukecup.jpg");
        gc.drawImage(image, 40.0, 540.0);
    }

    public static void main(String... args) {
        launch(args);
    }    
}
